@@ -467,12 +467,15 @@ StateFreqType ModelCodon::initGY94(bool fix_kappa, CodonKappaStyle kappa_style)
 
 
 void ModelCodon::computeRateAttributes() {
+    char symbols_protein[] = "ARNDCQEGHILKMFPSTWYVX"; // X for unknown AA
     int i, j, ts, tv;
     int nrates = getNumRateEntries();
     if (!rate_attr) {
         rate_attr = new int[nrates];
         memset(rate_attr, 0, sizeof(int)*nrates);
     }
+    char aa_cost_change[400];
+    memset(aa_cost_change, 10, sizeof(char)*400);
     for (i = 0; i < num_states; i++) {
         int *rate_attr_row = &rate_attr[i*num_states];
         if (phylo_tree->aln->isStopCodon(i)) {
@@ -488,12 +491,23 @@ void ModelCodon::computeRateAttributes() {
             int nuc1, nuc2;
             int attr = 0;
             ts = tv = 0;
-            if (phylo_tree->aln->genetic_code[i] == phylo_tree->aln->genetic_code[j])
+            int codoni = phylo_tree->aln->codon_table[i];
+            int codonj = phylo_tree->aln->codon_table[j];
+            if (phylo_tree->aln->genetic_code[codoni] == phylo_tree->aln->genetic_code[codonj])
                 attr |= CA_SYNONYMOUS;
             else
                 attr |= CA_NONSYNONYMOUS;
                 
-            if ((nuc1=i/16) != (nuc2=j/16)) {
+                
+            int nt_changes = ((codoni/16) != (codonj/16)) + (((codoni%16)/4) != ((codonj%16)/4)) + ((codoni%4) != (codonj%4));
+            int aa1 = strchr(symbols_protein, phylo_tree->aln->genetic_code[codoni]) - symbols_protein;
+            int aa2 = strchr(symbols_protein, phylo_tree->aln->genetic_code[codonj]) - symbols_protein;
+            assert(aa1 >= 0 && aa1 < 20 && aa2 >= 0 && aa2 < 20);
+            if (nt_changes < aa_cost_change[aa1*20+aa2]) {
+                aa_cost_change[aa1*20+aa2] = aa_cost_change[aa2*20+aa1] = nt_changes;
+            }
+            
+            if ((nuc1=codoni/16) != (nuc2=codonj/16)) {
                 if (abs(nuc1-nuc2)==2) { // transition 
                     attr |= CA_TRANSITION_1NT;
                     ts++;
@@ -502,7 +516,7 @@ void ModelCodon::computeRateAttributes() {
                     tv++;
                 }
             }
-            if ((nuc1=(i%16)/4) != (nuc2=(j%16)/4)) {
+            if ((nuc1=(codoni%16)/4) != (nuc2=(codonj%16)/4)) {
                 if (abs(nuc1-nuc2)==2) { // transition
                     attr |= CA_TRANSITION_2NT;
                     ts++;
@@ -511,7 +525,7 @@ void ModelCodon::computeRateAttributes() {
                     tv++;
                 }
             }
-            if ((nuc1=i%4) != (nuc2=j%4)) {
+            if ((nuc1=codoni%4) != (nuc2=codonj%4)) {
                 if (abs(nuc1-nuc2)==2) { // transition
                     attr |= CA_TRANSITION_3NT;
                     ts++;
@@ -530,6 +544,32 @@ void ModelCodon::computeRateAttributes() {
             rate_attr_row[j] = attr;
         }
     }
+    
+    if (verbose_mode >= VB_MAX) {
+
+        // make cost matrix fulfill triangular inequality
+        for (int k = 0; k < 20; k++)
+            for (i = 0; i < 20; i++)
+                for (j = 0; j < 20; j++)
+                    if (aa_cost_change[i*20+j] > aa_cost_change[i*20+k] + aa_cost_change[k*20+j])
+                        aa_cost_change[i*20+j] = aa_cost_change[i*20+k] + aa_cost_change[k*20+j];
+
+        cout << "cost matrix by number of nt changes for TNT use" << endl;
+        cout << "smatrix =1 (aa_nt_changes)";
+        for (i = 0; i < 19; i++)
+            for (j = i+1; j < 20; j++)
+                cout << " " << symbols_protein[i] << "/" << symbols_protein[j] << " " << (int)aa_cost_change[i*20+j];
+        cout << ";" << endl;
+        cout << 20 << endl;
+        for (i = 0; i < 20; i++) {
+            aa_cost_change[i*20+i] = 0;
+            for (j = 0; j < 20; j++)
+                cout << (int)aa_cost_change[i*20+j] << " ";
+            cout << endl;
+        }
+        
+    }
+    
 }
 
 void ModelCodon::combineRateNTFreq() {
@@ -541,15 +581,18 @@ void ModelCodon::combineRateNTFreq() {
         for (j = 0; j < num_states; j++)  {
             if (this_rate[j] == 0.0)
                 continue;
+
+            int codoni = phylo_tree->aln->codon_table[i];
+            int codonj = phylo_tree->aln->codon_table[j];
             int nuc1, nuc2;
                 
-            if ((nuc1=i/16) != (nuc2=j/16)) {
+            if ((nuc1=codoni/16) != (nuc2=codonj/16)) {
                 this_rate[j] *= ntfreq[nuc2];
             }
-            if ((nuc1=(i%16)/4) != (nuc2=(j%16)/4)) {
+            if ((nuc1=(codoni%16)/4) != (nuc2=(codonj%16)/4)) {
                 this_rate[j] *= ntfreq[nuc2+4];
             }
-            if ((nuc1=i%4) != (nuc2=j%4)) {
+            if ((nuc1=codoni%4) != (nuc2=codonj%4)) {
                 this_rate[j] *= ntfreq[nuc2+8];
             }
         }
@@ -589,8 +632,8 @@ void ModelCodon::readCodonModel(istream &in, bool reset_params) {
 		int nt3 = phylo_tree->aln->convertState(codons[i][2], SEQ_DNA);
 		if (nt1 > 3 || nt2 > 3 || nt3 > 3)
 			outError("Wrong codon triplet ", codons[i]);
-		state_map[i] = nt1*16+nt2*4+nt3;
-		if (phylo_tree->aln->isStopCodon(state_map[i]))
+		state_map[i] = phylo_tree->aln->non_stop_codon[nt1*16+nt2*4+nt3];
+		if (phylo_tree->aln->isStopCodon(state_map[i]) || state_map[i] == STATE_INVALID)
 			outError("Stop codon encountered");
 		if (verbose_mode >= VB_MAX)
 			cout << " " << codons[i] << " " << state_map[i];
@@ -939,9 +982,9 @@ double ModelCodon::optimizeParameters(double gradient_epsilon) {
 	// by BFGS algorithm
 	setVariables(variables);
 	setBounds(lower_bound, upper_bound, bound_check);
-//    if (phylo_tree->params->optimize_alg.find("BFGS-B") == string::npos)
-//        score = -minimizeMultiDimen(variables, ndim, lower_bound, upper_bound, bound_check, max(gradient_epsilon, TOL_RATE));
-//    else
+    if (phylo_tree->params->optimize_alg.find("BFGS-B") == string::npos)
+        score = -minimizeMultiDimen(variables, ndim, lower_bound, upper_bound, bound_check, max(gradient_epsilon, TOL_RATE));
+    else
         score = -L_BFGS_B(ndim, variables+1, lower_bound+1, upper_bound+1, max(gradient_epsilon, TOL_RATE));
 
 	bool changed = getVariables(variables);
