@@ -76,6 +76,12 @@ public:
 	 */
 	virtual int getNMixtures() { return 1; }
 
+ 	/**
+	 * @param cat mixture class
+	 * @return weight of a mixture model component
+	 */
+	virtual double getMixtureWeight(int cat) { return 1.0; }
+
 	/**
 		@return the number of rate entries, equal to the number of elements
 			in the upper-diagonal of the rate matrix (since model is reversible)
@@ -257,6 +263,14 @@ public:
 		return NULL;
 	}
 
+    /**
+     * compute the memory size for the model, can be large for site-specific models
+     * @return memory size required in bytes
+     */
+    virtual uint64_t getMemoryRequired() {
+    	return num_states*sizeof(double);
+    }
+
 	/*****************************************************
 		Checkpointing facility
 	*****************************************************/
