@@ -1638,7 +1638,7 @@ void PhyloTree::computePartialParsimonyFastSIMD(PhyloNeighbor *dad_branch, Phylo
         switch (nstates) {
         case 4:
             #ifdef _OPENMP
-            #pragma omp parallel for private (site) reduction(+: score) if(nsites>200)
+            #pragma omp parallel for private (site) reduction(+: score) if(nsites>num_threads*10)
             #endif
 			for (site = 0; site<nsites; site++) {
                 size_t offset = entry_size*site;
@@ -1666,7 +1666,7 @@ void PhyloTree::computePartialParsimonyFastSIMD(PhyloNeighbor *dad_branch, Phylo
 			break;
         default:
             #ifdef _OPENMP
-            #pragma omp parallel for private (site) reduction(+: score) if(nsites > 800/nstates)
+            #pragma omp parallel for private (site) reduction(+: score) if(nsites>num_threads*10)
             #endif
 			for (site = 0; site<nsites; site++) {
                 size_t offset = entry_size*site;
@@ -1730,7 +1730,7 @@ int PhyloTree::computeParsimonyBranchFastSIMD(PhyloNeighbor *dad_branch, PhyloNo
     switch (nstates) {
     case 4:
         #ifdef _OPENMP
-        #pragma omp parallel for private (site) reduction(+: score) if(nsites>200)
+        #pragma omp parallel for private (site) reduction(+: score) if(nsites>num_threads*10)
         #endif
 		for (site = 0; site < nsites; site++) {
             size_t offset = entry_size*site;
@@ -1749,7 +1749,7 @@ int PhyloTree::computeParsimonyBranchFastSIMD(PhyloNeighbor *dad_branch, PhyloNo
 		break;
     default:
         #ifdef _OPENMP
-        #pragma omp parallel for private (site) reduction(+: score) if(nsites > 800/nstates)
+        #pragma omp parallel for private (site) reduction(+: score) if(nsites>num_threads*10)
         #endif
 		for (site = 0; site < nsites; site++) {
             size_t offset = entry_size*site;
