@@ -23,15 +23,26 @@
 #include "superalignment.h"
 #include "phylosupertree.h"
 
-SuperAlignment::SuperAlignment()
- : Alignment() {}
+SuperAlignment::SuperAlignment() : Alignment() {
+    max_num_states = 0;
+}
 
-SuperAlignment::SuperAlignment(PhyloSuperTree *super_tree)
- : Alignment()
+SuperAlignment::SuperAlignment(PhyloSuperTree *super_tree) : Alignment()
 {
+    max_num_states = 0;
 	// first build taxa_index and partitions
 	int site, seq, nsite = super_tree->size();
 	PhyloSuperTree::iterator it;
+
+    // BUG FIX 2016-11-29: when merging partitions with -m TESTMERGE, sequence order is changed
+    // get the taxa names from existing tree
+    if (super_tree->root) {
+        super_tree->getTaxaName(seq_names);
+        taxa_index.resize(seq_names.size());
+        for (auto i = taxa_index.begin(); i != taxa_index.end(); i++)
+            i->resize(nsite, -1);
+    }
+        
 	for (site = 0, it = super_tree->begin(); it != super_tree->end(); it++, site++) {
 		partitions.push_back((*it)->aln);
 		int nseq = (*it)->aln->getNSeq();
@@ -189,7 +200,9 @@ Alignment *SuperAlignment::removeIdenticalSeq(string not_remove, bool keep_two,
 					removed_seqs.push_back(getSeqName(seq2));
 					target_seqs.push_back(getSeqName(seq1));
 					removed[seq2] = true;
-				}
+				} else {
+                    cout << "NOTE: " << getSeqName(seq2) << " is identical to " << getSeqName(seq1) << " but kept for subsequent analysis" << endl;
+                }
 				checked[seq2] = 1;
 				first_ident_seq = false;
 			}
@@ -212,6 +225,13 @@ Alignment *SuperAlignment::removeIdenticalSeq(string not_remove, bool keep_two,
 	return aln;
 }
 
+int SuperAlignment::checkAbsentStates(string msg) {
+    int count = 0;
+    for (auto it = partitions.begin(); it != partitions.end(); it++)
+        count += (*it)->checkAbsentStates("partition " + convertIntToString((it-partitions.begin())+1));
+    return count;
+}
+
 /*
 void SuperAlignment::checkGappySeq() {
 	int nseq = getNSeq(), part = 0, i;
@@ -260,7 +280,7 @@ void SuperAlignment::getSitePatternIndex(IntVector &pattern_index) {
 }
 
 void SuperAlignment::getPatternFreq(IntVector &pattern_freq) {
-	if (!isSuperAlignment()) outError("Internal error: ", __func__);
+	ASSERT(isSuperAlignment());
 	int offset = 0;
 	if (!pattern_freq.empty()) pattern_freq.resize(0);
 	for (vector<Alignment*>::iterator it = partitions.begin(); it != partitions.end(); it++) {
@@ -272,22 +292,78 @@ void SuperAlignment::getPatternFreq(IntVector &pattern_freq) {
 }
 
 void SuperAlignment::createBootstrapAlignment(Alignment *aln, IntVector* pattern_freq, const char *spec) {
-	if (!aln->isSuperAlignment()) outError("Internal error: ", __func__);
-	if (pattern_freq) outError("Unsupported yet.", __func__);
-	if (spec) outError("Unsupported yet.", __func__);
+	ASSERT(aln->isSuperAlignment());
 	Alignment::copyAlignment(aln);
 	SuperAlignment *super_aln = (SuperAlignment*) aln;
-	if (!partitions.empty()) outError("Internal error: ", __func__);
-	for (vector<Alignment*>::iterator it = super_aln->partitions.begin(); it != super_aln->partitions.end(); it++) {
-		Alignment *boot_aln = new Alignment;
-		boot_aln->createBootstrapAlignment(*it);
-		partitions.push_back(boot_aln);
-	}
+	ASSERT(partitions.empty());
+
+	if (spec && strncmp(spec, "GENE", 4) == 0) {
+		// resampling whole genes
+        partitions.resize(super_aln->partitions.size(), NULL);
+        int i, ptn;
+        for (i = 0; i < super_aln->partitions.size(); i++) {
+			int part = random_int(super_aln->partitions.size());
+            if (!partitions[part]) {
+                // allocate the partition
+                partitions[part] = new Alignment;
+                if (strncmp(spec,"GENESITE",8) == 0) {
+                    partitions[part]->createBootstrapAlignment(super_aln->partitions[part]);
+                } else
+                    partitions[part]->copyAlignment(super_aln->partitions[part]);
+            } else {
+                Alignment *newaln;
+                if (strncmp(spec,"GENESITE",8) == 0) {
+                    Alignment *newaln = new Alignment;
+                    newaln->createBootstrapAlignment(super_aln->partitions[part]);
+                } else
+                     newaln = super_aln->partitions[part];
+
+                for (ptn = 0; ptn < super_aln->partitions[part]->size(); ptn++)
+                    partitions[part]->at(ptn).frequency += newaln->at(ptn).frequency;
+                if (strncmp(spec,"GENESITE",8) == 0)
+                    delete newaln;
+            }
+        }
+
+        // fulfill genes that are missing
+        for (i = 0; i < partitions.size(); i++)
+            if (!partitions[i]) {
+                partitions[i] = new Alignment;
+                partitions[i]->copyAlignment(super_aln->partitions[i]);
+                // reset all frequency
+                for (ptn = 0; ptn < partitions[i]->size(); ptn++)
+                    partitions[i]->at(ptn).frequency = 0;
+            }
+
+        // fill up pattern_freq vector
+        if (pattern_freq) {
+            pattern_freq->resize(0);
+            for (i = 0; i < partitions.size(); i++)
+                for (ptn = 0; ptn < partitions[i]->size(); ptn++)
+                    pattern_freq->push_back(partitions[i]->at(ptn).frequency);
+        }
+    } else if (!spec) {
+		// resampling sites within genes
+        for (vector<Alignment*>::iterator it = super_aln->partitions.begin(); it != super_aln->partitions.end(); it++) {
+            Alignment *boot_aln = new Alignment;
+            if (pattern_freq) {
+                IntVector part_pattern_freq;
+                boot_aln->createBootstrapAlignment(*it, &part_pattern_freq);
+                pattern_freq->insert(pattern_freq->end(), part_pattern_freq.begin(), part_pattern_freq.end());
+            } else {
+                boot_aln->createBootstrapAlignment(*it);
+            }
+            partitions.push_back(boot_aln);
+        }
+    } else {
+        outError("Wrong -bsam, either -bsam GENE or -bsam GENESITE");
+    }
 	taxa_index = super_aln->taxa_index;
+    countConstSite();
 }
 
 void SuperAlignment::createBootstrapAlignment(IntVector &pattern_freq, const char *spec) {
-	if (!isSuperAlignment()) outError("Internal error: ", __func__);
+	ASSERT(isSuperAlignment());
 	int nptn = 0;
 	for (vector<Alignment*>::iterator it = partitions.begin(); it != partitions.end(); it++) {
 		nptn += (*it)->getNPattern();
@@ -298,39 +374,11 @@ void SuperAlignment::createBootstrapAlignment(IntVector &pattern_freq, const cha
 	pattern_freq.insert(pattern_freq.end(), internal_freq, internal_freq + nptn);
 	delete [] internal_freq;
 
-/*	if (spec && strncmp(spec, "GENE", 4) != 0) outError("Unsupported yet.", __func__);
-
-	int offset = 0;
-	if (!pattern_freq.empty()) pattern_freq.resize(0);
-
-	if (spec && strncmp(spec, "GENE", 4) == 0) {
-		// resampling whole genes
-		int nptn = 0;
-		IntVector part_pos;
-		for (vector<Alignment*>::iterator it = partitions.begin(); it != partitions.end(); it++) {
-			part_pos.push_back(nptn);
-			nptn += (*it)->getNPattern();
-		}
-		pattern_freq.resize(nptn, 0);
-		for (int i = 0; i < partitions.size(); i++) {
-			int part = random_int(partitions.size());
-			for (int j = 0; j < partitions[part]->getNPattern(); j++)
-				pattern_freq[j + part_pos[part]] += partitions[part]->at(j).frequency;
-		}
-	} else {
-		// resampling sites within genes
-		for (vector<Alignment*>::iterator it = partitions.begin(); it != partitions.end(); it++) {
-			IntVector freq;
-			(*it)->createBootstrapAlignment(freq);
-			pattern_freq.insert(pattern_freq.end(), freq.begin(), freq.end());
-			offset += freq.size();
-		}
-	}*/
 }
 
 
 void SuperAlignment::createBootstrapAlignment(int *pattern_freq, const char *spec, int *rstream) {
-	if (!isSuperAlignment()) outError("Internal error: ", __func__);
+	ASSERT(isSuperAlignment());
 //	if (spec && strncmp(spec, "GENE", 4) != 0) outError("Unsupported yet. ", __func__);
 
 	if (spec && strncmp(spec, "GENE", 4) == 0) {
@@ -374,7 +422,7 @@ void SuperAlignment::createBootstrapAlignment(int *pattern_freq, const char *spe
  * shuffle alignment by randomizing the order of sites
  */
 void SuperAlignment::shuffleAlignment() {
-	if (!isSuperAlignment()) outError("Internal error: ", __func__);
+	ASSERT(isSuperAlignment());
 	for (vector<Alignment*>::iterator it = partitions.begin(); it != partitions.end(); it++) {
 		(*it)->shuffleAlignment();
 	}
@@ -540,8 +588,7 @@ Alignment *SuperAlignment::concatenateAlignments(IntVector &ids) {
 	SeqType sub_type = SEQ_UNKNOWN;
 	for (i = 0; i < ids.size(); i++) {
 		int id = ids[i];
-		if (id < 0 || id >= partitions.size())
-			outError("Internal error ", __func__);
+		ASSERT(id >= 0 && id < partitions.size());
 		if (nstates == 0) nstates = partitions[id]->num_states;
 		if (sub_type == SEQ_UNKNOWN) sub_type = partitions[id]->seq_type;
 		if (sub_type != partitions[id]->seq_type)
@@ -569,6 +616,12 @@ Alignment *SuperAlignment::concatenateAlignments(IntVector &ids) {
     aln->pattern_index.clear();
     aln->STATE_UNKNOWN = partitions[ids[0]]->STATE_UNKNOWN;
     aln->genetic_code = partitions[ids[0]]->genetic_code;
+    if (aln->seq_type == SEQ_CODON) {
+    	aln->codon_table = new char[aln->num_states];
+    	memcpy(aln->codon_table, partitions[ids[0]]->codon_table, aln->num_states);
+    	aln->non_stop_codon = new char[strlen(aln->genetic_code)];
+    	memcpy(aln->non_stop_codon, partitions[ids[0]]->non_stop_codon, strlen(aln->genetic_code));
+    }
 
     int site = 0;
     for (i = 0; i < ids.size(); i++) {
@@ -599,3 +652,50 @@ Alignment *SuperAlignment::concatenateAlignments(IntVector &ids) {
 
 	return aln;
 }
+
+void SuperAlignment::countConstSite() {
+    num_informative_sites = 0;
+    max_num_states = 0;
+    frac_const_sites = 0;
+    frac_invariant_sites = 0;
+    size_t nsites = 0;
+    for (vector<Alignment*>::iterator it = partitions.begin(); it != partitions.end(); it++) {
+        (*it)->countConstSite();
+        num_informative_sites += (*it)->num_informative_sites;
+        if ((*it)->num_states > max_num_states)
+            max_num_states = (*it)->num_states;
+        nsites += (*it)->getNSite();
+        frac_const_sites += (*it)->frac_const_sites * (*it)->getNSite();
+        frac_invariant_sites += (*it)->frac_invariant_sites * (*it)->getNSite();
+    }
+    frac_const_sites /= nsites;
+    frac_invariant_sites /= nsites;
+}
+
+void SuperAlignment::orderPatternByNumChars() {
+    const int UINT_BITS = sizeof(UINT)*8;
+    int maxi = (num_informative_sites+UINT_BITS-1)/UINT_BITS;
+    pars_lower_bound = new UINT[maxi+1];
+    memset(pars_lower_bound, 0, (maxi+1)*sizeof(UINT));
+    int part, nseq = getNSeq(), npart = partitions.size();
+    
+    // compute ordered_pattern
+    ordered_pattern.clear();
+    UINT sum_scores[npart];
+    for (part  = 0; part != partitions.size(); part++) {
+        partitions[part]->orderPatternByNumChars();
+        // partial_partition
+        for (vector<Pattern>::iterator pit = partitions[part]->ordered_pattern.begin(); pit != partitions[part]->ordered_pattern.end(); pit++) {
+            Pattern pattern(*pit);
+            pattern.resize(nseq); // maximal unknown states
+            for (int j = 0; j < nseq; j++)
+                if (taxa_index[j][part] >= 0)
+                    pattern[j] = (*pit)[taxa_index[j][part]];
+                else
+                    pattern[j] = partitions[part]->STATE_UNKNOWN;
+            ordered_pattern.push_back(pattern);
+        }
+        sum_scores[part] = partitions[part]->pars_lower_bound[0];
+    }
+    // TODO compute pars_lower_bound (lower bound of pars score for remaining patterns)
+}
