@@ -425,8 +425,20 @@ struct NNIInfo {
     int iqpnni_iteration;
 };
 
+/*
+    0           = 80386 instruction set
+    1  or above = SSE (XMM) supported by CPU (not testing for O.S. support)
+    2  or above = SSE2
+    3  or above = SSE3
+    4  or above = Supplementary SSE3 (SSSE3)
+    5  or above = SSE4.1
+    6  or above = SSE4.2
+    7  or above = AVX supported by CPU and operating system
+    8  or above = AVX2
+    9  or above = AVX512F
+*/
 enum LikelihoodKernel {
-	LK_EIGEN, LK_EIGEN_SSE
+	LK_386, LK_SSE, LK_SSE2, LK_SSE3, LK_SSSE3, LK_SSE41, LK_SSE42, LK_AVX, LK_AVX_FMA, LK_AVX512
 };
 
 enum LhMemSave {
@@ -1450,9 +1462,6 @@ public:
      */
     LikelihoodKernel SSE;
 
-    /** TRUE to not use AVX even available in CPU, default: FALSE */
-    int lk_no_avx;
-
     /** TRUE for safe numerical scaling (per category; used for large trees), default: FALSE */
     bool lk_safe_scaling;
 
@@ -1807,6 +1816,9 @@ public:
 	 */
 	bool print_bootaln;
 
+    /** TRUE to print bootstrapped site frequency for e.g. PMSF */
+    bool print_boot_site_freq;
+
 	/** true to print sub alignments of super alignment, default: false */
 	bool print_subaln;
 
@@ -1961,6 +1973,9 @@ void outWarning(const char *warn);
 void outWarning(string warn);
 
 
+/** safe version of std::getline to deal with files from different platforms */ 
+std::istream& safeGetline(std::istream& is, std::string& t);
+
 /*--------------------------------------------------------------*/
 /*--------------------------------------------------------------*/
 
@@ -2317,11 +2332,11 @@ int random_int(int n, int *rstream = NULL);
 double random_double(int *rstream = NULL);
 
 template <class T>
-void my_random_shuffle (T first, T last)
+void my_random_shuffle (T first, T last, int *rstream = NULL)
 {
 	int n = last - first;
 	for (int i=n-1; i>0; --i) {
-		swap (first[i],first[random_int(i+1)]);
+		swap (first[i],first[random_int(i+1, rstream)]);
 	}
 }
 
