@@ -1,8 +1,8 @@
 /****************************  vectorf128.h   *******************************
 * Author:        Agner Fog
 * Date created:  2012-05-30
-* Last modified: 2015-11-27
-* Version:       1.20
+* Last modified: 2016-09-27
+* Version:       1.23
 * Project:       vector classes
 * Description:
 * Header file defining floating point vector classes as interface to 
@@ -30,14 +30,22 @@
 *
 * For detailed instructions, see VectorClass.pdf
 *
-* (c) Copyright 2012 - 2015 GNU General Public License http://www.gnu.org/licenses
+* (c) Copyright 2012 - 2016 GNU General Public License http://www.gnu.org/licenses
 *****************************************************************************/
 #ifndef VECTORF128_H
 #define VECTORF128_H
 
+#if defined _MSC_VER && _MSC_VER >= 1800
+// solve problem with ambiguous overloading of pow function in Microsoft math.h:
+// make sure math.h is included first rather than last
+#include <math.h>
+#endif 
+
 #include "vectori128.h"  // Define integer vectors
 
-
+#ifdef VCL_NAMESPACE
+namespace VCL_NAMESPACE {
+#endif
 
 /*****************************************************************************
 *
@@ -131,11 +139,11 @@ public:
     operator __m128() const {
         return xmm;
     }
-#if defined (__clang__) && CLANG_VERSION < 30900 || defined(__apple_build_version__)
-#define FIX_CLANG_VECTOR_ALIAS_AMBIGUITY  // clang 3.3 - 3.5 has silent conversion between intrinsic vector types. 
-                                          // I expected this to be fixed in version 3.4 but it still exists!
+#if defined (__clang__) /* && CLANG_VERSION < xxxxx */ || defined(__apple_build_version__)
+#define FIX_CLANG_VECTOR_ALIAS_AMBIGUITY  // clang 3.3 has silent conversion between intrinsic vector types. 
+                                          // I expected this to be fixed in version 3.4 but it still exists in version 3.9!
                                           // http://llvm.org/bugs/show_bug.cgi?id=17164
-                                          // Problem: The version number is not consistent across platforms
+                                          // Additional problem: The version number is not consistent across platforms
                                           // The Apple build has different version numbers. Too bad!
                                           // http://llvm.org/bugs/show_bug.cgi?id=12643
 
@@ -246,12 +254,14 @@ static inline Vec4fb andnot(Vec4fb const & a, Vec4fb const & b) {
 
 // horizontal_and. Returns true if all bits are 1
 static inline bool horizontal_and (Vec4fb const & a) {
-    return horizontal_and(Vec128b(_mm_castps_si128(a)));
+    return _mm_movemask_ps(a) == 0x0F; 
+    //return horizontal_and(Vec128b(_mm_castps_si128(a)));
 }
 
 // horizontal_or. Returns true if at least one bit is 1
 static inline bool horizontal_or (Vec4fb const & a) {
-    return horizontal_or(Vec128b(_mm_castps_si128(a)));
+    return _mm_movemask_ps(a) != 0;
+    //return horizontal_or(Vec128b(_mm_castps_si128(a)));
 }
 
 
@@ -414,12 +424,14 @@ static inline Vec2db andnot(Vec2db const & a, Vec2db const & b) {
 
 // horizontal_and. Returns true if all bits are 1
 static inline bool horizontal_and (Vec2db const & a) {
-    return horizontal_and(Vec128b(_mm_castpd_si128(a)));
+    return _mm_movemask_pd(a) == 3;
+    //return horizontal_and(Vec128b(_mm_castpd_si128(a)));
 }
 
 // horizontal_or. Returns true if at least one bit is 1
 static inline bool horizontal_or (Vec2db const & a) {
-    return horizontal_or(Vec128b(_mm_castpd_si128(a)));
+    return _mm_movemask_pd(a) != 0;
+    //return horizontal_or(Vec128b(_mm_castpd_si128(a)));
 }
 
 
@@ -491,9 +503,9 @@ public:
         case 1:
             xmm = _mm_load_ss(p); break;
         case 2:
-            xmm = _mm_castpd_ps(_mm_load_sd((double*)p)); break;
+            xmm = _mm_castpd_ps(_mm_load_sd((double const*)p)); break;
         case 3:
-            t1 = _mm_castpd_ps(_mm_load_sd((double*)p));
+            t1 = _mm_castpd_ps(_mm_load_sd((double const*)p));
             t2 = _mm_load_ss(p + 2);
             xmm = _mm_movelh_ps(t1, t2); break;
         case 4:
@@ -1894,7 +1906,7 @@ static inline Vec2d pow(Vec2d const & a, Const_int_t<n>) {
 
 // avoid unsafe optimization in function round
 #if defined(__GNUC__) && !defined(__INTEL_COMPILER) && !defined(__clang__) && INSTRSET < 5
-static inline Vec4f round(Vec4f const & a) __attribute__ ((optimize("-fno-unsafe-math-optimizations")));
+static inline Vec2d round(Vec2d const & a) __attribute__ ((optimize("-fno-unsafe-math-optimizations")));
 #elif defined (FLOAT_CONTROL_PRECISE_FOR_ROUND)
 #pragma float_control(push) 
 #pragma float_control(precise,on)
@@ -2618,4 +2630,8 @@ static inline Vec2db to_Vec2db(uint8_t x) {
     return Vec2db(to_Vec2qb(x));
 }
 
+#ifdef VCL_NAMESPACE
+}
+#endif
+
 #endif // VECTORF128_H
